--- old/usr/src/tools/smatch/src/smatch_helper.c	Mon Aug  5 08:38:33 2019
+++ new/usr/src/tools/smatch/src/smatch_helper.c	Mon Aug  5 08:38:33 2019
@@ -81,14 +81,26 @@
 	return state;
 }
 
+struct smatch_state *merge_str_state(struct smatch_state *s1, struct smatch_state *s2)
+{
+	if (!s1->name || !s2->name)
+		return &merged;
+	if (strcmp(s1->name, s2->name) == 0)
+		return s1;
+	return &merged;
+}
+
 struct smatch_state *alloc_state_expr(struct expression *expr)
 {
 	struct smatch_state *state;
 	char *name;
 
-	state = __alloc_smatch_state(0);
 	expr = strip_expr(expr);
 	name = expr_to_str(expr);
+	if (!name)
+		return NULL;
+
+	state = __alloc_smatch_state(0);
 	state->name = alloc_sname(name);
 	free_string(name);
 	state->data = expr;
@@ -166,7 +178,7 @@
 
 		deref = expr->deref;
 		op = deref->op;
-		if (op == '*') {
+		if (deref->type == EXPR_PREOP && op == '*') {
 			struct expression *unop = strip_expr(deref->unop);
 
 			if (unop->type == EXPR_PREOP && unop->op == '&') {
@@ -173,9 +185,9 @@
 				deref = unop->unop;
 				op = '.';
 			} else {
-				deref = deref->unop;
-				if (!is_pointer(deref))
+				if (!is_pointer(deref) && !is_pointer(deref->unop))
 					op = '.';
+				deref = deref->unop;
 			}
 		}
 
@@ -527,7 +539,7 @@
 		if (sym)
 			*sym = tmp;
 		if (vsl)
-			*vsl = expr_to_vsl(expr);
+			add_var_sym(vsl, name, tmp);
 		return name;
 	}
 	free_string(name);
@@ -869,8 +881,52 @@
 			 expr->member->name);
 		return alloc_string(buf);
 	}
-	if (!sym->ident)
-		return NULL;
+	if (!sym->ident) {
+		struct expression *deref;
+		char *full, *outer;
+		int len;
+
+		/*
+		 * If we're in an anonymous struct then maybe we can find an
+		 * outer struct name to use as a name.  This code should be
+		 * recursive and cleaner.  I am not very proud of it.
+		 *
+		 */
+
+		deref = expr->deref;
+		if (deref->type != EXPR_DEREF || !deref->member)
+			return NULL;
+		sym = get_type(deref->deref);
+		if (!sym || sym->type != SYM_STRUCT || !sym->ident)
+			return NULL;
+
+		full = expr_to_str(expr);
+		if (!full)
+			return NULL;
+		deref = deref->deref;
+		if (deref->type == EXPR_PREOP && deref->op == '*')
+			deref = deref->unop;
+		outer = expr_to_str(deref);
+		if (!outer) {
+			free_string(full);
+			return NULL;
+		}
+		len = strlen(outer);
+		if (strncmp(outer, full, len) != 0) {
+			free_string(full);
+			free_string(outer);
+			return NULL;
+		}
+		if (full[len] == '-' && full[len + 1] == '>')
+			len += 2;
+		if (full[len] == '.')
+			len++;
+		snprintf(buf, sizeof(buf), "(struct %s)->%s", sym->ident->name, full + len);
+		free_string(outer);
+		free_string(full);
+
+		return alloc_string(buf);
+	}
 	snprintf(buf, sizeof(buf), "(struct %s)->%s", sym->ident->name, expr->member->name);
 	return alloc_string(buf);
 }
@@ -1054,6 +1110,34 @@
 	return 0;
 }
 
+int op_remove_assign(int op)
+{
+	switch (op) {
+	case SPECIAL_ADD_ASSIGN:
+		return '+';
+	case SPECIAL_SUB_ASSIGN:
+		return '-';
+	case SPECIAL_MUL_ASSIGN:
+		return '*';
+	case SPECIAL_DIV_ASSIGN:
+		return '/';
+	case SPECIAL_MOD_ASSIGN:
+		return '%';
+	case SPECIAL_AND_ASSIGN:
+		return '&';
+	case SPECIAL_OR_ASSIGN:
+		return '|';
+	case SPECIAL_XOR_ASSIGN:
+		return '^';
+	case SPECIAL_SHL_ASSIGN:
+		return SPECIAL_LEFTSHIFT;
+	case SPECIAL_SHR_ASSIGN:
+		return SPECIAL_RIGHTSHIFT;
+	default:
+		return op;
+	}
+}
+
 int expr_equiv(struct expression *one, struct expression *two)
 {
 	struct symbol *one_sym = NULL;
